#Remember that each folder contain their own records
# and the LAT2 function inside each folder is for only available records in that folder
# nad they differ from LAT2 in Dynamic subpackage that loads ant AT2 inside the current folder
# for any new folder of records only add __init__ and LAT2.py from one of the existing folders
 